@@ -1,5 +1,5 @@
 #
-# Copyright 2014-2023 Ghent University
+# Copyright 2014-2025 Ghent University
 #
 # This file is part of vsc-install,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -36,16 +36,17 @@ Running python setup.py test will pick this up and do its magic
 
 import optparse
 import os
-import pkg_resources
 import pprint
 import re
 import sys
+import pkg_resources
+from pathlib import Path
 from prospector.run import Prospector
 from prospector.config import ProspectorConfig
 
 from distutils import log
 from vsc.install.ci import (
-    JENKINSFILE, TOX_INI, GITHUB_ACTIONS, gen_jenkinsfile, gen_tox_ini,
+    JENKINSFILE, TOX_INI, GITHUB_ACTIONS, RUFF_TOML, gen_ruff_toml, gen_jenkinsfile, gen_tox_ini,
     gen_github_action, parse_vsc_ci_cfg, ENABLE_GITHUB_ACTIONS)
 from vsc.install.headers import check_header
 from vsc.install.shared_setup import vsc_setup
@@ -82,9 +83,13 @@ PROSPECTOR_WHITELIST = [
     'bad-indentation',
     'bad-whitespace',
     'bare-except',
+    'consider-using-dict-comprehension',
+    'consider-using-f-string',
+    'consider-using-set-comprehension',
     'consider-using-with', # By using 'with' the release of the allocated res is ensured in the case of an exception.
     'dangerous-default-value',
     'duplicate-key',  # when a key appears twice in a dict definition
+    'implicit-str-concat',
     'import-star-module-level',  # Import * only allowed at module level
     'inconsistent-return-statements', # Either all or no return statements in a function should return an expression.
     'indexing-exception',  # indexing exceptions doesn't work in python3, use Exc.args[index] instead (but why?)
@@ -103,14 +108,18 @@ PROSPECTOR_WHITELIST = [
     'redefine-in-handler',  # except A, B -> except (A, B)
     'redefined-builtin',
     'reimported',
+    'super-with-arguments',
     'syntax-error',
     'trailing-whitespace',
     'undefined',
+    'unidiomatic-typecheck',
+    'unnecessary-pass',
     'unnecessary-semicolon',
     'unpacking-in-except', # will stop working in python3
     'unused-argument',
     'unused-import',
     'unused-variable',
+    'useless-object-inheritance',
 ]
 
 # Prospector commandline options (positional path is added automatically)
@@ -159,7 +168,7 @@ def run_prospector(base_dir, clear_ignore_patterns=False):
 
     # add/set REPO_BASE_DIR as positional path
     sys.argv.append(base_dir)
-    log.debug("prospector commandline %s" % sys.argv)
+    log.debug("prospector commandline %s", sys.argv)
 
     config = ProspectorConfig()
     # prospector will sometimes wrongly autodetect django
@@ -181,11 +190,11 @@ def run_prospector(base_dir, clear_ignore_patterns=False):
 
     # Enable pylint Python3 compatibility tests:
     config.profile.pylint['options']['enable'] = 'python3'
-    log.debug("prospector argv = %s" % sys.argv)
-    log.debug("prospector profile from config = %s" % vars(config.profile))
+    log.debug("prospector argv = %s", sys.argv)
+    log.debug("prospector profile from config = %s", vars(config.profile))
 
     prospector.execute()
-    log.debug("prospector profile form prospector = %s" % vars(prospector.config.profile))
+    log.debug("prospector profile form prospector = %s", vars(prospector.config.profile))
 
     # map yields a generator object in Python 3, but since we want to iterate over the whitelist/blacklist
     # multiple times, we need to make sure it's a list (since you can only iterate once over a generator)
@@ -193,25 +202,53 @@ def run_prospector(base_dir, clear_ignore_patterns=False):
     whitelist = list(map(re.compile, PROSPECTOR_WHITELIST))
 
     failures = []
+
+    # message.as_dict() has been removed after prospector 1.8.0
+    # same for location.as_dict()
+    # mimic old behaviour
+    def prospector_message_as_dict(message):
+        if hasattr(message, "as_dict"):
+            return message.as_dict()
+        else:
+            return {
+                "source": message.source,
+                "code": message.code,
+                "location": prospector_location_as_dict(message.location),
+                "message": message.message,
+            }
+
+    def prospector_location_as_dict(location):
+        if hasattr(location, "as_dict"):
+            return location.as_dict()
+        else:
+            return {
+                "path": location.path,
+                "module": location.module,
+                "function": location.function,
+                "line": location.line,
+                "character": location.character,
+            }
+
     for msg in prospector.get_messages():
         # example msg.as_dict():
         #  {'source': 'pylint', 'message': 'Missing function docstring', 'code': 'missing-docstring',
         #   'location': {'function': 'TestHeaders.test_check_header.lgpl', 'path': u'headers.py',
         #                'line': 122, 'character': 8, 'module': 'headers'}}
-        log.debug("prospector message %s" % msg.as_dict())
+        log.debug("prospector message %s", prospector_message_as_dict(msg))
 
         if any([bool(reg.search(msg.code) or reg.search(msg.message)) for reg in blacklist]):
             continue
 
         if any([bool(reg.search(msg.code) or reg.search(msg.message)) for reg in whitelist]):
-            failures.append(msg.as_dict())
+            failures.append(prospector_message_as_dict(msg))
 
     # The following is still the case in 3.6. seems to be fixed in 3.9:
     # There is some ugly monkeypatch code in pylint
     #     (or logilab if no recent enough pylint is installed)
     # Make sure the original is restored
     # (before any errors are reported; no need to put this in setUp/tearDown)
-    optparse.HelpFormatter.expand_default = orig_expand_default
+    if sys.version_info < (3,9):
+        optparse.HelpFormatter.expand_default = orig_expand_default
 
     return failures
 
@@ -220,27 +257,25 @@ def check_autogenerated_ci_config_file(testcase_instance, ci_cfg_fn, expected_co
     """Test whether specified CI configuration file is in place, and was auto-generated by vsc-install."""
 
     testcase_instance.assertTrue(os.path.exists(ci_cfg_fn))
-    with open(ci_cfg_fn) as fh:
-        current_contents = fh.read()
+    current_contents = Path(ci_cfg_fn).read_text(encoding='utf8')
 
     # allow differing Jenkinsfile if Jenkinsfile.NOT_AUTOGENERATED_YET exists (and likewise for tox.ini)
     # if contents are equal to what's generated, purposely fail the check
     ci_cfg_ignore_fn = ci_cfg_fn + '.NOT_AUTOGENERATED_YET'
 
     if os.path.exists(ci_cfg_ignore_fn):
-        error_msg = "Found %s, so contents of %s is expected to be different " % (ci_cfg_ignore_fn, ci_cfg_fn)
-        error_msg += "from what vsc-install generates (but it's not!): %s" % current_contents
+        error_msg = f"Found {ci_cfg_ignore_fn}, so contents of {ci_cfg_fn} is expected to be different "
+        error_msg += f"from what vsc-install generates (but it's not!): {current_contents}"
         testcase_instance.assertNotEqual(current_contents, expected_contents, error_msg)
 
         # if the ignore file exists, it should contain the URL of a vsc-install issue
-        with open(ci_cfg_ignore_fn) as ignore_file:
-            txt = ignore_file.read()
-            regex = re.compile(r'https://github.com/hpcugent/vsc-install/issues/[0-9]+')
-            error_msg = "Pattern '%s' does not match in contents of %s: %s" % (regex.pattern, ci_cfg_ignore_fn, txt)
-            testcase_instance.assertTrue(bool(regex.search(txt)), error_msg)
+        txt = Path(ci_cfg_ignore_fn).read_text(encoding='utf8')
+        regex = re.compile(r'https://github.com/hpcugent/vsc-install/issues/[0-9]+')
+        error_msg = f"Pattern '{regex.pattern}' does not match in contents of {ci_cfg_ignore_fn}: {txt}"
+        testcase_instance.assertTrue(bool(regex.search(txt)), error_msg)
     else:
-        error_msg = "Contents of %s does not match expected contents, " % ci_cfg_fn
-        error_msg += "you should run 'python -m vsc.install.ci' again to re-generate %s" % ci_cfg_fn
+        error_msg = f"Contents of {ci_cfg_fn} does not match expected contents, "
+        error_msg += f"you should run 'python -m vsc.install.ci' again to re-generate {ci_cfg_fn}"
         testcase_instance.assertEqual(current_contents, expected_contents, error_msg)
 
 
@@ -266,22 +301,22 @@ class CommonTest(TestCase):
         """Cleanup after running a test."""
         self.orig_sys_argv = sys.argv
         self.setup = vsc_setup()
-        super(CommonTest, self).setUp()
+        super().setUp()
 
     def tearDown(self):
         """Cleanup after running a test."""
         sys.argv = self.orig_sys_argv
-        super(CommonTest, self).tearDown()
+        super().tearDown()
 
     def _import(self, pkg):
         try:
             __import__(pkg)
-        except ImportError as e:
+        except ImportError as err:
             log.debug("__path__ %s",
-                      ["%s = %s" % (name, getattr(mod, '__path__', 'None')) for name, mod in sys.modules.items()])
-            self.assertFalse(e, msg="import %s failed sys.path %s exception %s" % (pkg, sys.path, e))
+                      [f"{name} = {getattr(mod, '__path__', 'None')}" for name, mod in sys.modules.items()])
+            self.assertFalse(err, msg=f"import {pkg} failed sys.path {sys.path} exception {err}")
 
-        self.assertTrue(pkg in sys.modules, msg='%s in sys.modules after import' % pkg)
+        self.assertTrue(pkg in sys.modules, msg=f'{pkg} in sys.modules after import')
 
     def test_import_packages(self):
         """Try to import each namespace"""
@@ -289,10 +324,10 @@ class CommonTest(TestCase):
             self._import(pkg)
 
             if self.CHECK_HEADER:
-                for fn in self.setup.files_in_packages()['packages'][pkg]:
-                    self.assertFalse(check_header(os.path.join(self.setup.REPO_BASE_DIR, fn),
+                for fun in self.setup.files_in_packages()['packages'][pkg]:
+                    self.assertFalse(check_header(os.path.join(self.setup.REPO_BASE_DIR, fun),
                                      script=False, write=False),
-                                     msg='check_header of %s' % fn)
+                                     msg=f'check_header of {fun}')
 
     def test_import_modules(self):
         """Try to import each module"""
@@ -309,13 +344,13 @@ class CommonTest(TestCase):
 
             if self.CHECK_HEADER:
                 self.assertFalse(check_header(os.path.join(self.setup.REPO_BASE_DIR, scr), script=True, write=False),
-                                 msg='check_header of %s' % scr)
+                                 msg=f'check_header of {scr}')
 
     def test_prospector(self):
         """Test prospector failures"""
 
         failures = run_prospector(self.setup.REPO_BASE_DIR)
-        self.assertFalse(failures, "prospector failures: %s" % pprint.pformat(failures))
+        self.assertFalse(failures, f"prospector failures: {pprint.pformat(failures)}")
 
     def test_jenkinsfile(self):
         """Test whether Jenkinsfile is in place, and was auto-generated by vsc-install."""
@@ -330,3 +365,7 @@ class CommonTest(TestCase):
         vsc_ci_cfg = parse_vsc_ci_cfg()
         if os.path.exists(GITHUB_ACTIONS) or vsc_ci_cfg[ENABLE_GITHUB_ACTIONS]:
             check_autogenerated_ci_config_file(self, GITHUB_ACTIONS, gen_github_action())
+
+    def test_ruff_toml(self):
+        """Test whether ruff.toml is in place, and was auto-generated by vsc-install."""
+        check_autogenerated_ci_config_file(self, RUFF_TOML, gen_ruff_toml())
