@@ -1,5 +1,5 @@
 #
-# Copyright 2015-2023 Ghent University
+# Copyright 2015-2025 Ghent University
 #
 # This file is part of vsc-install,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -42,6 +42,7 @@ import os
 import re
 import sys
 
+from pathlib import Path
 from datetime import date
 from vsc.install.shared_setup import SHEBANG_BIN_BASH, SHEBANG_ENV_PYTHON, log, vsc_setup
 
@@ -91,10 +92,9 @@ def get_header(filename, script=False):
     """
 
     if not os.path.isfile(filename):
-        raise Exception('get_header filename %s not found' % filename)
+        raise ValueError(f'get_header filename {filename} not found')
 
-    with open(filename) as fh:
-        txt = fh.read()
+    txt = Path(filename).read_text(encoding='utf8')
 
     blocks = HEADER_REGEXP.split(txt)
     if len(blocks) == 1:
@@ -102,7 +102,7 @@ def get_header(filename, script=False):
         blocks = ['', '']
     elif blocks[0] != '':
         # the block before the begin of text is always empty
-        raise Exception('unexpected non-empty block with get_header %s: %s' % (filename, blocks))
+        raise ValueError(f'unexpected non-empty block with get_header {filename}: {blocks}')
 
     header = blocks[1]
 
@@ -128,20 +128,20 @@ def gen_license_header(license_name, **kwargs):
         name: project name
         url: project url
     """
-    template_name = "%s_TEMPLATE" % license_name.replace('+', '_plus_')
+    template_name = f"{license_name.replace('+', '_plus_')}_TEMPLATE"
     template = globals().get(template_name, None)
     if template is None:
-        raise Exception('gen_license_header cannot find template name %s' % template_name)
+        raise ValueError(f'gen_license_header cannot find template name {template_name}')
 
     found = False
-    for github_organ in institute_details.keys():
+    for github_organ, details in institute_details.items():
         if github_organ in kwargs.get('url', '').lower():
-            kwargs.update(institute_details[github_organ])
+            kwargs.update(details)
             found = True
             break
 
     if not found:
-        raise Exception('Unable to find a known github organization in url %s' % kwargs.get('url'))
+        raise ValueError(f"Unable to find a known github organization in url {kwargs.get('url')}")
 
     return template.format(**kwargs)
 
@@ -172,10 +172,8 @@ def begin_end_from_header(header):
 
 
 def _write(filename, content):
-    """Simple wrapper around open().write for unittesting"""
-    with open(filename, 'w') as fh:
-        fh.write(content)
-
+    """Simple wrapper around Path().write_text() for unittesting"""
+    Path(filename).write_text(content, encoding='utf8')
 
 def check_header(filename, script=False, write=False):
     """
@@ -200,7 +198,7 @@ def check_header(filename, script=False, write=False):
         # scripts must have an appropriate shebang
         shebang = orig_shebang
         file_ext = os.path.splitext(filename)[1]
-        log.info("Shebang found in %s (ext: %s): %s", filename, file_ext, shebang)
+        log.info(f"Shebang found in {filename} (ext: {file_ext}): {shebang}")
 
         if file_ext == '.py':
             if shebang != SHEBANG_ENV_PYTHON:
@@ -271,13 +269,12 @@ def check_header(filename, script=False, write=False):
         new_header = gen_header
 
     if orig_header != new_header:
-        log.info("Diff orig_header vs new_header for %s\n" % filename + ''.join(nicediff(orig_header, new_header)))
+        log.info("Diff orig_header vs new_header for %s\n", filename + ''.join(nicediff(orig_header, new_header)))
         changed = True
 
     if write and changed:
-        log.info('write enabled and different header. Going to modify file %s' % filename)
-        with open(filename) as fh:
-            wholetext = fh.read()
+        log.info('write enabled and different header. Going to modify file %s', filename)
+        wholetext = Path(filename).read_text(encoding='utf8')
         _write(filename, new_header + wholetext[header_end_pos:])
 
     # return different or not
@@ -383,5 +380,5 @@ if __name__ == '__main__':
         args.pop(-1)
 
     for fn in args:
-        log.info('Going to check_header for file %s (is_script=%s)' % (fn, is_script))
+        log.info(f'Going to check_header for file {fn} (is_script={is_script})')
         check_header(fn, script=is_script, write=True)
